__name__ = 'factory_djoy'
__version__ = '1.0.1'

__author__ = 'James Cooke'
__copyright__ = '2018, {}'.format(__author__)

__description__ = 'Factories for Django, creating valid model instances every time.'
__email__ = 'github@jamescooke.info'
